import { InternalProvider } from './InternalProvider';
import { Factory, ILifetime } from './ILifetime';
import { DependencyProvider, NameSelector } from './types';
declare type LifetimeProvider<T, E> = new (container: Container<E>, dependency: DependencyProvider<T, E>, providerName: string, factoryFunction: Factory<T, E>) => ILifetime<T, E>;
declare type ProviderProvider<E> = {
    prototype: E;
    name: string;
    new (): E;
};
declare type DependencyOptions<T, E> = {
    factory?: Factory<T, E>;
    selector?: NameSelector<T, E>;
};
declare type DependencyGetter<T, E> = string;
declare type ActualProvider<E> = E & InternalProvider<E>;
export declare class Container<E> {
    readonly template: E;
    private readonly providerLookup;
    private readonly dependencyToProvider;
    private provider?;
    constructor(ProviderTemplate: ProviderProvider<E>);
    addSingleton<T>(Dependency: DependencyProvider<T, E>, options?: DependencyOptions<T, E>): void;
    tryAddSingleton<T>(Dependency: DependencyProvider<T, E>, options?: DependencyOptions<T, E>): void;
    addTransient<T>(Dependency: DependencyProvider<T, E>, options?: DependencyOptions<T, E>): void;
    tryAddTransient<T>(Dependency: DependencyProvider<T, E>, options?: DependencyOptions<T, E>): void;
    add<T>(Lifetime: LifetimeProvider<T, E>, Dependency: DependencyProvider<T, E>, options?: DependencyOptions<T, E>): void;
    tryAdd<T>(Lifetime: LifetimeProvider<T, E>, Dependency: DependencyProvider<T, E>, { factory, selector }?: DependencyOptions<T, E>): boolean;
    /**
     * Calling this there is no guarantee that it can provide an instance
     */
    get<T>(item: DependencyGetter<T, E>): ILifetime<T, E> | undefined;
    preBuildValidate(): void;
    build(): ActualProvider<E>;
}
export {};
