"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InternalProvider = void 0;
var DependencyError_1 = require("./DependencyError");
var utils_1 = require("./utils");
var types_1 = require("./types");
var InternalProvider = /** @class */ (function () {
    function InternalProvider(container) {
        this._container = container;
    }
    InternalProvider.prototype.get = function (nameSelector) {
        var name = nameSelector((0, utils_1.properties)(this._container.template));
        try {
            return this.getByString(name);
        }
        catch (e) {
            var error = e;
            if (error.type === types_1.DependencyErrorType.Circular)
                throw new DependencyError_1.DependencyError({ type: types_1.DependencyErrorType.Circular, lifetime: name });
            throw e;
        }
    };
    InternalProvider.prototype.getByString = function (name) {
        try {
            var result = this._container.get(name);
            if (result)
                return result.provide();
        }
        catch (e) {
            var error = e;
            if (error.message === 'Maximum call stack size exceeded')
                throw new DependencyError_1.DependencyError({ type: types_1.DependencyErrorType.Circular, lifetime: name });
            throw e;
        }
        throw new DependencyError_1.DependencyError({ type: types_1.DependencyErrorType.Existence, lifetime: name });
    };
    InternalProvider.prototype.validate = function () {
        var keys = Object.keys(this._container.template);
        var unresolved = [];
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            try {
                this.getByString(key);
            }
            catch (e) {
                unresolved.push(e);
            }
        }
        if (unresolved.length === 0)
            return;
        throw new DependencyError_1.DependencyMultiError(types_1.DependencyMultiErrorType.Validation, unresolved);
    };
    return InternalProvider;
}());
exports.InternalProvider = InternalProvider;
